"""断面算定がOKとなる鉄骨断面を求める

『SS7』で計算を繰り返し行い、
断面算定がNGの場合、断面のサイズアップを行い、断面算定がOKとなる鉄骨断面を求めます。

Copyright (C) 2024 UNION SYSTEM Inc.

"""

import calc_ss7 as ss7
import csv_operation as co
import shutil
import os
import time


def prepare_file_paths(input_csv_name: str) -> tuple[str, str, str]:
    """
    指定された入力CSVファイル名を基に、関連するファイルパスを準備し、返します。

    Args:
        input_csv_name (str): 入力として使用されるCSVファイルの名前。

    Raises:
        FileNotFoundError: 指定された入力CSVファイルが見つからない場合に発生します。

    Returns:
        tuple[str, str, str]: コピーされた入力CSVファイルのパス、結果CSVファイルのパス、物件データフォルダのパスを含むタプル。
    """
    try:
        # 現在のスクリプトファイルの絶対パスを取得
        script_directory = os.path.dirname(os.path.abspath(__file__))
        # スクリプトのディレクトリから一階層上のディレクトリを取得
        parent_directory = os.path.dirname(script_directory)

        # 入力csvのフォルダのパス
        input_folder_path = os.path.join(parent_directory, "input")
        # 元の入力csvファイルのパス
        base_input_csv_path = os.path.join(input_folder_path, input_csv_name)
        # 入力CSVファイルの存在を確認
        if not os.path.exists(base_input_csv_path):
            raise FileNotFoundError(
                f"入力CSVファイルが見つかりません: {base_input_csv_path}"
            )

        # 出力するフォルダのパス
        output_folder_path = os.path.join(
            parent_directory, "output", input_csv_name.replace(".csv", "")
        )
        # 物件フォルダの存在を確認し、必要に応じて作成
        if not os.path.exists(output_folder_path):
            os.makedirs(output_folder_path)

        # 入力CSVを出力フォルダへコピー
        input_csv_path = os.path.join(output_folder_path, "input.csv")
        shutil.copy(base_input_csv_path, input_csv_path)

        # 結果csvの出力先を指定
        result_csv_path = os.path.join(output_folder_path, "result.csv")
        # 『SS7』データの出力先を指定
        ss7data_path = os.path.join(output_folder_path, "Ss7Data.ikn")

        return input_csv_path, result_csv_path, ss7data_path

    except FileNotFoundError as e:
        print(f"エラー: {e}")
    except OSError as e:
        # OS関連のエラー（例: ディレクトリの作成やファイルのコピーに失敗した場合）
        print(f"ファイル操作に失敗しました: {e}")
    except Exception as e:
        # その他の予期せぬエラー
        print(f"予期せぬエラーが発生しました: {e}")
    return "", "", ""


def perform_analysis(
    input_csv_name: str = "input.csv",
    max_count: int = 100,
) -> None:
    """
     断面算定がNGとなる部材がなくなるまで計算を繰り返す関数。

    Args:
        input_csv_name (str): 入力CSVファイルの名前。デフォルト="input.csv"
        max_count (float): 最大計算回数。デフォルト=100
    """

    # 入力CSV、結果CSV、物件データフォルダのパスを準備します。
    input_csv_path, result_csv_path, ss7data_path = prepare_file_paths(input_csv_name)
    if input_csv_path == "":
        return

    # 結果名
    result_name: str = "結果1"
    # 計算する項目名
    calc_item: str = "断面算定"
    # 回数
    count: int = 1
    # 合計実行時間を追跡
    total_time: float = 0.0
    # 区切り文字の指定
    separator: str = "～" * 30
    short_separator: str = "-" * 30

    # 処理のメインループ開始
    for count in range(1, max_count + 1):
        start_time: float = time.time()  # 現在の時間を取得
        print(separator)
        print(f"{count}回目開始。\n")

        # 入力csvからデータを作成し、計算を実行、結果csvを出力
        ss7ana = ss7.run_analysis(
            input_csv_path,
            ss7data_path,
            result_csv_path,
            result_name,
            calc_item,
        )
        if not ss7ana:
#            print("断面算定まで計算できませんでした。")
            return

        # 計算終了後の時間を取得し、実行時間を計算
        end_time: float = time.time()
        elapsed_time: float = end_time - start_time
        total_time += elapsed_time  # 合計時間に加算

        # 断面算定がNGとなっている符号の一覧を取得
        ng_gir = co.get_ng_fugo(
            result_csv_path,
            "S梁検定比一覧",
        )
        ng_col = co.get_ng_fugo(
            result_csv_path,
            "S柱検定比一覧",
        )


        # 計算を続行するか（断面算定をすべて満足しているか）の判定
        if not ng_gir and not ng_col:
            print(separator)
            print("全てのS梁・S柱の断面算定を満足しました。\n")
            break

        elif count == max_count:
            print("最大計算回数に到達しました。計算を終了します。")
            break

        print(short_separator)
        print(f"断面算定NG符号数：梁={len(ng_gir)}  柱={len(ng_col)}。計算を続行します。\n")

        # 断面算定がNGとなっている部材の鉄骨断面を取得する
        co.set_sgirsect(
            input_csv_path,
            ng_gir,
        )
        co.set_scolsect(
            input_csv_path,
            ng_col,
        )

        # 断面算定がNGとなっている部材の鉄骨断面を変更する
        update_num = co.update_sect(
            input_csv_path,
            ng_gir,
            ng_col,
        )
        if update_num <= 0:
            print("更新できる断面がありませんでした。\n")
            break

        # 入力csvを更新
        co.update_sgirsect(
            input_csv_path,
            ng_gir,
        )
        co.update_scolsect(
            input_csv_path,
            ng_col,
        )

        print(f"{count}回目：{elapsed_time:.2f}秒\n")

        count += 1
    print(separator)
    print(f"合計実行時間：{total_time:.2f}秒")
    print(separator)
