"""『SS7』で計算を実行する関数

『SS7』を用いて入力csvから物件を作成・計算し、結果csvを出力します。

Copyright (C) 2024 UNION SYSTEM Inc.

"""

from Python import Ss7Python as Cmd


def run_analysis(
    input_csv_path: str,
    ss7data_path: str,
    result_csv_path: str,
    result_name: str,
    calc_item: str,
) -> bool:
    """
    SS7を使用して計算を実行し、結果をCSVファイルに出力する関数。

    Args:
        input_csv_path (str): 計算用の入力CSVファイルのパス。
        ss7data_path (str): 計算対象の物件ファイルのパス。
        result_csv_path (str): 計算結果を出力するCSVファイルのパス。
        result_name (str): 結果名。
        calc_item (str): 計算する項目名。

    Returns:
        bool: 計算が無事に終了したか
    """
    try:
        # SS7のセッションを開始し、ログファイルをクリアする
        Cmd.Init()
        Cmd.Start("1.1.1.19", 1)  # 1=ログファイルをクリアする

        # CSVからSS7データを作成
        Cmd.CreateDataCsv(input_csv_path, ss7data_path, 1)

        # エラー情報を取得する
        err = Cmd.GetLastError()

        if not err.IsOK():
            print(err.GetErrorMessage())
            raise Exception

        # 物件データを開く
        data = Cmd.Open(ss7data_path, 1, 3)
        if not data:
            # エラー情報を取得する
            err = Cmd.GetLastError()
            print(err.GetErrorMessage())
            raise Exception

        # 計算を実行
        print("計算実行中...")
        data.Calculate(result_name, calc_item)

        # エラー情報を取得する
        err = Cmd.GetLastError()

        if not err.IsOK():
            print(err.GetErrorMessage())
            data.Close(2)  # 保存せずに閉じる
            raise Exception

        if not data.IsCalcEnd(result_name, calc_item):
            print(f"{calc_item}まで計算を実行できませんでした。")
            raise Exception

        print("計算が終了しました。\n")

        # 計算結果を取得
        result_data = data.GetResultData(result_name)
        if not result_data:
            # エラー情報を取得する
            err = Cmd.GetLastError()
            print(err.GetErrorMessage())
            data.Close(2)  # 保存せずに閉じる
            raise Exception

        # CSVファイルに結果をエクスポート
        export_items = "7.1.3.S梁;7.1.4.S柱"
        result_data.ExportResultCsv(export_items, result_csv_path, 1, 1, 1)

        if not Cmd.GetLastError().IsOK():
            # エラー情報を取得する
            err = Cmd.GetLastError()
            print(err.GetErrorMessage())

        # データを保存して閉じる
        data.Close(1)  # 変更を保存して閉じる
        return True

    except Exception:
        return False
