"""csvの操作を行う関数

Copyright (C) 2024 UNION SYSTEM Inc.

"""

import csv


def get_ng_fugo( result_csv_path, search_keys )  -> list[str, str, "", "", ""]:
    """
    断面算定がNGとなっている符号の一覧をリストに入れて返します。

    Args:
        result_csv_path: 結果CSVファイルの名前。
        search_keys: 検索キー。

    Returns:
        list[str, str, "", "", ""]: 断面算定NGの符号リスト（[層/階名, 符号名, "", "", ""]）。

    """

    # CSVファイルを開いて読み込む
    with open(result_csv_path, "r", encoding="cp932") as file:
        ng_list = []
        reader = csv.reader(file)
        btarget: bool = False  # 対象テーブル名を読み込んだかどうかのフラグ
        bdata: bool = False  # データセクションに入ったかどうかのフラグ
        table_name: str = ""
        count_m: int = 0  # 曲げの検討箇所数
        count_q: int = 0  # せん断の検討箇所数
        for row in reader:
            if len(row) > 0:
                # テーブル名とケースの行をチェック
                if "name=" in row[0]:
                    table_name = row[0].replace("name=", "")
                    if table_name in search_keys:  # テーブル名が検索キーの値と一致する場合
                        btarget = True
                    continue

                # データセクションの開始行をチェック
                elif row[0] == "<data>":
                    bdata = True
                    continue

                # テーブル名が読み込んだ次の行にあるヘッダの解釈
                if btarget and count_m == 0 and count_q == 0:
                    section_m = False
                    section_q = False
                    for c in row:
                        if c == "M":
                            section_m = True
                            count_m += 1
                            continue
                        if c == "Q":
                            section_q = True
                            count_q += 1
                            continue
                        if len(c) > 0:
                            if section_m and section_q:
                                break
                        else:
                            if section_q:
                                count_q += 1
                                continue
                            if section_m:
                                count_m += 1
                                continue

                    continue

                # テーブル名が読み込まれ、データセクションに入っている場合
                if btarget and bdata:
                    # 曲げとせん断の検定比を取得し、1.00を超えている箇所があればリストに登録する
                    name_fl: str = ""
                    name_fugo: str = ""
                    count: int = 0
                    for element in row:
                        count += 1
                        if count == 1:
                            name_fl = element
                            continue
                        if count == 2:
                            name_fugo = element
                            continue
                        if count <= 2+count_m:
                            # 曲げ検定比
                            if len(element) > 0:
                                ratio = float(element)
                                if ratio > 1.0:
                                    # 断面算定NG -> リストに追加
                                    ng_list.append([name_fl, name_fugo, "", "", ""])
                                    break
                            continue
                        if count <= 2+count_m+count_q:
                            # せん断検定比
                            if len(element) > 0:
                                ratio = float(element)
                                if ratio > 1.0:
                                    # 断面算定NG -> リストに追加
                                    ng_list.append([name_fl, name_fugo, "", "", ""])
                                    break
                            continue
                        if count > 2+count_m+count_q:
                            break

            else:
                if btarget and bdata:
                    # 読み込み終了
                    break
                else:
                    # 空行があれば、フラグをリセット
                    btarget = False
                    bdata = False
                    table_name = ""
                    count_m = 0
                    count_q = 0

        return ng_list


def set_sgirsect( input_csv_path, gir_list )  -> None:
    """
    断面算定がNGとなっているS梁の鉄骨断面をセットします。

    Args:
        input_csv_path: 結果CSVファイルの名前。
        gir_list: NG梁符号のリスト（[層名, 符号名, 形状, TYPE, 鉄骨断面]）。

    """

    # CSVファイルを開いて読み込む
    with open(input_csv_path, "r", encoding="cp932") as file:
        reader = csv.reader(file)
        btarget: bool = False  # 対象テーブル名を読み込んだかどうかのフラグ
        bdata: bool = False  # データセクションに入ったかどうかのフラグ
        table_name: str = ""
        for row in reader:
            if len(row) > 0:
                # テーブル名とケースの行をチェック
                if "name=" in row[0]:
                    table_name = row[0].replace("name=", "")
                    if table_name in "S梁断面":  # テーブル名が検索キーの値と一致する場合
                        btarget = True
                    continue

                # データセクションの開始行をチェック
                elif row[0] == "<data>":
                    bdata = True
                    continue

                # テーブル名が読み込まれ、データセクションに入っている場合
                if btarget and bdata:
                    name_fl: str = ""
                    name_fugo: str = ""
                    name_prefix: str = ""
                    name_form: str = ""
                    name_type: str = ""
                    name_sect: str = ""
                    count: int = 0
                    for element in row:
                        count += 1
                        if count == 1:
                            name_fl = element
                            continue
                        if count == 2:
                            name_fugo = element
                            continue
                        if count == 3:
                            name_prefix = element
                            continue
                        if count == 6:
                            name_form = element
                            continue
                        if count == 7:
                            name_sect = element
                            continue
                        if count == 8:
                            name_type = element
                            continue
                        if count >= 9:
                            break

                    # NG部材の符号かチェック
                    if name_prefix == "-":
                        # 添字なし（符号名と層名で判定）
                        for gir in gir_list:
                            if gir[0] == name_fl and gir[1] == name_fugo:
                                gir[2] = name_form
                                gir[3] = name_type
                                gir[4] = name_sect
                                break
                    else:
                        gir_fugo = name_prefix + name_fugo
                        for gir in gir_list:
                            if gir[1] == gir_fugo:
                                gir[2] = name_form
                                gir[3] = name_type
                                gir[4] = name_sect
                                break

            else:
                if btarget and bdata:
                    # 読み込み終了
                    break
                else:
                    # 空行があれば、フラグをリセット
                    btarget = False
                    bdata = False
                    table_name = ""


def update_sgirsect( input_csv_path, gir_list )  -> None:
    """
    断面算定がNGとなっているS梁の鉄骨断面を更新します。

    Args:
        input_csv_path: 結果CSVファイルの名前。
        gir_list: NG梁符号のリスト（[層名, 符号名, 形状, TYPE, 鉄骨断面]）。

    """

    updated_rows = []  # 更新後の行を格納するリスト
    with open(input_csv_path, "r", encoding="cp932") as file:
        reader = csv.reader(file)
        btarget: bool = False  # 対象テーブル名を読み込んだかどうかのフラグ
        bdata: bool = False  # データセクションに入ったかどうかのフラグ
        table_name: str = ""
        for row in reader:
            if len(row) > 0:
                # テーブル名とケースの行をチェック
                if "name=" in row[0]:
                    table_name = row[0].replace("name=", "")
                    if table_name in "S梁断面":  # テーブル名が検索キーの値と一致する場合
                        btarget = True
                    updated_rows.append(row)  # 更新された行をリストに追加
                    continue

                # データセクションの開始行をチェック
                elif row[0] == "<data>":
                    bdata = True
                    updated_rows.append(row)  # 更新された行をリストに追加
                    continue

                # テーブル名が読み込まれ、データセクションに入っている場合
                if btarget and bdata:
                    name_fl: str = ""
                    name_fugo: str = ""
                    name_prefix: str = ""
                    count: int = 0
                    for element in row:
                        count += 1
                        if count == 1:
                            name_fl = element
                            continue
                        if count == 2:
                            name_fugo = element
                            continue
                        if count == 3:
                            name_prefix = element
                            continue
                        if count >= 4:
                            break

                    # NG部材の符号かチェック
                    if name_prefix == "-":
                        # 添字なし（符号名と層名で判定）
                        for gir in gir_list:
                            if gir[0] == name_fl and gir[1] == name_fugo:
                                if len(gir[4]) > 0:
                                    row[6] = gir[4]
                                    row[7] = gir[3]
                                    row[8] = gir[4]
                                    row[9] = gir[3]
                                    row[10] = gir[4]
                                    row[11] = gir[3]
                                break
                    else:
                        gir_fugo = name_prefix + name_fugo
                        for gir in gir_list:
                            if gir[1] == gir_fugo:
                                if len(gir[4]) > 0:
                                    row[6] = gir[4]
                                    row[7] = gir[3]
                                    row[8] = gir[4]
                                    row[9] = gir[3]
                                    row[10] = gir[4]
                                    row[11] = gir[3]
                                break

            else:
                # 空行があれば、フラグをリセット
                btarget = False
                bdata = False
                table_name = ""

            updated_rows.append(row)  # 更新された行をリストに追加

    # 更新されたデータでCSVファイルを上書き
    with open(input_csv_path, "w", newline="", encoding="cp932") as file:
        writer = csv.writer(file)
        writer.writerows(updated_rows)



def set_scolsect( input_csv_path, col_list )  -> None:
    """
    断面算定がNGとなっているS柱の鉄骨断面をセットします。

    Args:
        input_csv_path: 結果CSVファイルの名前。
        col_list: NG柱符号のリスト（[階名, 符号名, 形状, TYPE, 鉄骨断面]）。

    """

    # CSVファイルを開いて読み込む
    with open(input_csv_path, "r", encoding="cp932") as file:
        reader = csv.reader(file)
        btarget: bool = False  # 対象テーブル名を読み込んだかどうかのフラグ
        bdata: bool = False  # データセクションに入ったかどうかのフラグ
        table_name: str = ""
        for row in reader:
            if len(row) > 0:
                # テーブル名とケースの行をチェック
                if "name=" in row[0]:
                    table_name = row[0].replace("name=", "")
                    if table_name in "S柱断面":  # テーブル名が検索キーの値と一致する場合
                        btarget = True
                    continue

                # データセクションの開始行をチェック
                elif row[0] == "<data>":
                    bdata = True
                    continue

                # テーブル名が読み込まれ、データセクションに入っている場合
                if btarget and bdata:
                    name_fl: str = ""
                    name_fugo: str = ""
                    name_prefix: str = ""
                    name_form: str = ""
                    name_type: str = ""
                    name_sect: str = ""
                    count: int = 0
                    for element in row:
                        count += 1
                        if count == 1:
                            name_fl = element
                            continue
                        if count == 2:
                            name_fugo = element
                            continue
                        if count == 3:
                            name_prefix = element
                            continue
                        if count == 4:
                            name_form = element
                            continue
                        if count == 5:
                            name_sect = element
                            continue
                        if count == 6:
                            name_type = element
                            continue
                        if count == 7:
                            if name_form == "Ｉ":
                                name_sect = element
                            continue
                        if count == 8:
                            if name_form == "Ｉ":
                                name_type = element
                            continue
                        if count >= 9:
                            break

                    # NG部材の符号かチェック
                    if name_prefix == "-":
                        # 添字なし（符号名と階名で判定）
                        for col in col_list:
                            if col[0] == name_fl and col[1] == name_fugo:
                                col[2] = name_form
                                col[3] = name_type
                                col[4] = name_sect
                                break
                    else:
                        col_fugo = name_prefix + name_fugo
                        for col in col_list:
                            if col[1] == col_fugo:
                                col[2] = name_form
                                col[3] = name_type
                                col[4] = name_sect
                                break

            else:
                if btarget and bdata:
                    # 読み込み終了
                    break
                else:
                    # 空行があれば、フラグをリセット
                    btarget = False
                    bdata = False
                    table_name = ""


def update_scolsect( input_csv_path, col_list )  -> None:
    """
    断面算定がNGとなっているS柱の鉄骨断面を更新します。

    Args:
        input_csv_path: 結果CSVファイルの名前。
        col_list: NG柱符号のリスト（[階名, 符号名, 形状, TYPE, 鉄骨断面]）。

    """

    updated_rows = []  # 更新後の行を格納するリスト
    with open(input_csv_path, "r", encoding="cp932") as file:
        reader = csv.reader(file)
        btarget: bool = False  # 対象テーブル名を読み込んだかどうかのフラグ
        bdata: bool = False  # データセクションに入ったかどうかのフラグ
        table_name: str = ""
        for row in reader:
            if len(row) > 0:
                # テーブル名とケースの行をチェック
                if "name=" in row[0]:
                    table_name = row[0].replace("name=", "")
                    if table_name in "S柱断面":  # テーブル名が検索キーの値と一致する場合
                        btarget = True
                    updated_rows.append(row)  # 更新された行をリストに追加
                    continue

                # データセクションの開始行をチェック
                elif row[0] == "<data>":
                    bdata = True
                    updated_rows.append(row)  # 更新された行をリストに追加
                    continue

                # テーブル名が読み込まれ、データセクションに入っている場合
                if btarget and bdata:
                    name_fl: str = ""
                    name_fugo: str = ""
                    name_prefix: str = ""
                    count: int = 0
                    for element in row:
                        count += 1
                        if count == 1:
                            name_fl = element
                            continue
                        if count == 2:
                            name_fugo = element
                            continue
                        if count == 3:
                            name_prefix = element
                            continue
                        if count >= 4:
                            break

                    # NG部材の符号かチェック
                    if name_prefix == "-":
                        # 添字なし（符号名と階名で判定）
                        for col in col_list:
                            if col[0] == name_fl and col[1] == name_fugo:
                                if len(col[4]) > 0:
                                    if col[2] == "Ｉ":
                                        row[6] = col[4]
                                        row[7] = col[3]
                                    else:
                                        row[4] = col[4]
                                        row[5] = col[3]
                                break
                    else:
                        col_fugo = name_prefix + name_fugo
                        for col in col_list:
                            if col[1] == col_fugo:
                                if len(col[4]) > 0:
                                    if col[2] == "Ｉ":
                                        row[6] = col[4]
                                        row[7] = col[3]
                                    else:
                                        row[4] = col[4]
                                        row[5] = col[3]
                                break

            else:
                # 空行があれば、フラグをリセット
                btarget = False
                bdata = False
                table_name = ""

            updated_rows.append(row)  # 更新された行をリストに追加

    # 更新されたデータでCSVファイルを上書き
    with open(input_csv_path, "w", newline="", encoding="cp932") as file:
        writer = csv.writer(file)
        writer.writerows(updated_rows)



def update_sect( input_csv_path, ng_gir, ng_col )  -> int:
    """
    断面算定がNGとなっている部材の鉄骨断面を変更します。

    Args:
        input_csv_path: 結果CSVファイルの名前。
        ng_gir: NG梁符号のリスト（[層名, 符号名, 形状, TYPE, 鉄骨断面]）。
        ng_col: NG柱符号のリスト（[階名, 符号名, 形状, TYPE, 鉄骨断面]）。

    Returns:
        int: 鉄骨断面を変更した部材の数。

    """

    # 鋼材データを読み込む
    # 検索キーを定義
    search_keys = {
        "Ｈ形鋼",
        "角形鋼管",
        "円形鋼管",
    }
    table: dict[tuple[str], list[list[str]]] = {}
    # CSVファイルを開いて読み込む
    with open(input_csv_path, "r", encoding="cp932") as file:
        reader = csv.reader(file)
        btarget: bool = False  # 対象テーブル名を読み込んだかどうかのフラグ
        bdata: bool = False  # データセクションに入ったかどうかのフラグ
        table_name: str = ""
        for row in reader:
            if len(row) > 0:
                # テーブル名とケースの行をチェック
                if "name=" in row[0]:
                    table_name = row[0].replace("name=", "")
                    if table_name in search_keys:  # テーブル名が検索キーの値と一致する場合
                        btarget = True
                    continue

                # データセクションの開始行をチェック
                elif row[0] == "<data>":
                    bdata = True
                    table[table_name] = []
                    continue

                # テーブル名が読み込まれ、データセクションに入っている場合
                if btarget and bdata:
                    table[table_name].append(row)

            else:
                # 空行があれば、フラグをリセット
                btarget = False
                bdata = False
                table_name = ""

    update_num: int = 0
    # NG梁符号の断面を更新する
    for gir in ng_gir:
        if gir[2] == "断面性能":
            continue

        state: int = 0   # 0=探索中、1=更新中、2=更新完了
        for records in table["Ｈ形鋼"]:
            # 飛び番のデータは飛ばす
            if len(records[1]) == 0:
                continue
            if len(records[2]) == 0:
                continue
            if len(records[3]) == 0:
                continue
            if len(records[4]) == 0:
                continue
            if len(records[5]) == 0:
                continue
            if len(records[6]) == 0:
                continue
            if len(records[7]) == 0:
                continue

            sect: str = ""
            if records[7] == "0":
                sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]+"x"+records[6]
            else:
                sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]+"x"+records[6]+"x"+records[7]

            if state == 1:
                # 断面をセット
                gir[3] = records[1]
                gir[4] = sect
                update_num += 1
                state = 2
                break

            if gir[3] == records[1] and gir[4] == sect:
                state = 1
                gir[3] = ""
                gir[4] = ""     # 断面をいったん初期化
                continue
        
        if state < 2:
            return -1       # 断面が更新されずリストの最後まで達した

    # NG柱符号の断面を更新する
    for col in ng_col:
        if col[2] == "断面性能":
            continue

        state: int = 0   # 0=探索中、1=更新中、2=更新完了
        # 角形鋼管の場合
        if col[2] == "□":
            for records in table["角形鋼管"]:
                # 飛び番のデータは飛ばす
                if len(records[1]) == 0:
                    continue
                if len(records[2]) == 0:
                    continue
                if len(records[3]) == 0:
                    continue
                if len(records[4]) == 0:
                    continue
                if len(records[5]) == 0:
                    continue
                if len(records[6]) == 0:
                    continue
                if len(records[7]) == 0:
                    continue

                sect: str = ""
                if records[7] == "0":
                    if records[5] == records[6]:
                        sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]
                    else:
                        sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]+"x"+records[6]
                else:
                    if records[5] == records[6]:
                        sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]+"x"+records[7]
                    else:
                        sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]+"x"+records[6]+"x"+records[7]

                if state == 1:
                    # 断面をセット
                    col[3] = records[1]
                    col[4] = sect
                    update_num += 1
                    state = 2
                    break

                if col[3] == records[1] and col[4] == sect:
                    state = 1
                    col[3] = ""
                    col[4] = ""     # 断面をいったん初期化
                    continue
            
            if state < 2:
                return -1       # 断面が更新されずリストの最後まで達した

        # 円形鋼管の場合
        if col[2] == "○":
            for records in table["円形鋼管"]:
                # 飛び番のデータは飛ばす
                if len(records[1]) == 0:
                    continue
                if len(records[2]) == 0:
                    continue
                if len(records[3]) == 0:
                    continue
                if len(records[4]) == 0:
                    continue

                sect: str = records[2]+"-"+records[3]+"x"+records[4]

                if state == 1:
                    # 断面をセット
                    col[3] = records[1]
                    col[4] = sect
                    update_num += 1
                    state = 2
                    break

                if col[3] == records[1] and col[4] == sect:
                    state = 1
                    col[3] = ""
                    col[4] = ""     # 断面をいったん初期化
                    continue

            if state < 2:
                return -1       # 断面が更新されずリストの最後まで達した

        # Ｈ形鋼の場合
        for records in table["Ｈ形鋼"]:
            # 飛び番のデータは飛ばす
            if len(records[1]) == 0:
                continue
            if len(records[2]) == 0:
                continue
            if len(records[3]) == 0:
                continue
            if len(records[4]) == 0:
                continue
            if len(records[5]) == 0:
                continue
            if len(records[6]) == 0:
                continue
            if len(records[7]) == 0:
                continue

            sect: str = ""
            if records[7] == "0":
                sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]+"x"+records[6]
            else:
                sect = records[2]+"-"+records[3]+"x"+records[4]+"x"+records[5]+"x"+records[6]+"x"+records[7]

            if state == 1:
                # 断面をセット
                col[3] = records[1]
                col[4] = sect
                update_num += 1
                state = 2
                break

            if col[3] == records[1] and col[4] == sect:
                state = 1
                col[3] = ""
                col[4] = ""     # 断面をいったん初期化
                continue

        if state < 2:
            return -1       # 断面が更新されずリストの最後まで達した

    return update_num

